from typing import List

import pydash as __

from . import channel_manager
from .types import NotificationChannel, Notification

__all__ = ['SNSNotification', 'Notification', 'NotificationChannel']


class SNSNotification:
    def __init__(self, **kwargs):
        self._cache_send_channels = {}
        self._configs = __.clone_deep(kwargs)

    def send(self, notification: Notification):
        if callable(notification.via):
            channels = notification.via()
        else:
            channels = notification.via
        for channel in self._get_channels(channels):
            channel.send(notification)

    def reset_cache(self):
        self._cache_send_channels = {}

    def _get_channels(self, via_channels: List[str]) -> List[NotificationChannel]:
        results = []
        for key in via_channels:
            if key in self._cache_send_channels:
                channel = self._cache_send_channels[key]
                if channel is not None:
                    results.append(channel)
            else:
                channel = channel_manager.create_channel(key, self._configs)
                self._cache_send_channels[key] = channel
                if channel is not None:
                    results.append(channel)

        return results



from typing import Union, Callable, Dict, Optional, List

from jinja2 import Template

DictVar = Dict[str, str]
DictVarOrFunc = Union[DictVar, Callable[[], DictVar]]

__all__ = []


def render(template: Optional[str] = None,
           template_file: Optional[str] = None,
           variables: Dict[str, str] = {}):
    tpl_str = ''
    if template is not None:
        tpl_str = template
    if template_file is not None:
        with open(template_file, 'r') as f:
            tpl_str = f.read()

    return Template(tpl_str).render(**variables)


try:
    from SNSNotifications.channels.mail import Mailable

    __all__ += ['MailableNotification']


    class MailableNotification:
        email_variables: DictVarOrFunc = {}

        @property
        def email_template(self) -> Optional[str]:
            return None

        @property
        def email_template_file(self) -> Optional[str]:
            return None

        @property
        def email_to_addr(self) -> List[str]:
            raise NotImplementedError()

        @property
        def email_subject(self) -> str:
            raise NotImplementedError()

        @property
        def attachments(self) -> Optional[list]:
            return None

        def to_mail(self):
            return Mailable(to_addr=self.email_to_addr,
                            subject=self.email_subject,
                            attachments=self.attachments,
                            html=render(self.email_template, self.email_template_file, self.email_variables))
except ImportError:
    pass

try:
    from SNSNotifications.channels.slack import SlackMessage

    __all__ += ['SlackableNotification']


    class SlackableNotification:
        slack_variables: DictVarOrFunc = {}

        @property
        def slack_channel(self) -> Optional[str]:
            return None

        @property
        def slack_template(self) -> Optional[str]:
            return None

        @property
        def slack_mentions(self) -> Optional[List[str]]:
            return None

        def to_slack(self):
            return SlackMessage(text=render(self.slack_template, None, self.slack_variables),
                                mentions=self.slack_mentions,
                                channel=self.slack_channel)
except ImportError:
    pass



from typing import Optional, List


class Mailable:
    def __init__(self,
                 subject: str,
                 to_addr: List[str],
                 cc: Optional[List[str]] = None,
                 bcc: Optional[List[str]] = None,
                 from_addr: Optional[str] = None,
                 from_name: Optional[str] = None,
                 reply_to: Optional[str] = None,
                 plain: str = None,
                 html: str = None,
                 attachments: list = None):
        self.subject = subject
        self.to_addr = to_addr
        self.from_addr = from_addr
        self.from_name = from_name
        self.cc = cc
        self.bcc = bcc
        self.reply_to = reply_to
        self.plain = plain
        self.html = html
        self.attachments = attachments



from typing import Union, List, Set, Callable

__all__ = ['NotificationChannel', 'Notification']


class Notification:
    via: Union[List, Set, Callable[[], List[str]]] = []


class NotificationChannel:

    def send(self, notification: Notification):
        raise NotImplementedError()

    def channel_identify(self) -> str:
        raise NotImplementedError()

    def can_send(self, notification: Notification) -> bool:
        convert_fn_name = "to_{}".format(self.channel_identify())
        if not hasattr(notification, convert_fn_name):
            return False
        convert_fn = getattr(notification, convert_fn_name)
        return callable(convert_fn)



from typing import Optional, List

__all__ = ['SlackMessage']


class SlackMessage:
    def __init__(self, text: str = None,
                 blocks: Optional[List] = None,
                 has_link=False,
                 mentions: Optional[List[str]] = None,
                 channel=None):
        self.text = text
        self.blocks = blocks
        self.has_link = has_link
        self.mentions = mentions

        self.channel = channel
        if self.has_mentions:
            self.has_link = True
        if self.text is None:
            self.text = ''

    @property
    def has_mentions(self) -> bool:
        return self.mentions is not None and len(self.mentions) > 0

    @property
    def has_blocks(self):
        return self.blocks is not None and len(self.blocks) > 0

    def with_mentions(self, message_data):
        mentions = self._mention_text()
        if isinstance(message_data, str):
            return '{} {}'.format(mentions, message_data)
        if isinstance(message_data, list):
            return [{'type': 'section', 'text': {"type": "mrkdwn", "text": mentions}}] + message_data
        return message_data

    def _mention_text(self):
        mentions = map(lambda x: '@{}'.format(x), self.mentions)
        return ', '.join(mentions)



from .transports.smtp import SmtpTransport


class TransportManager:
    def create_smtp_driver(self, host: str, port: str,
                           username: str = None, password: str = None,
                           from_address: str = 'test@mail.com', from_name: str = 'Test',
                           encryption: bool = False,
                           **kwargs):
        transport = SmtpTransport(host, int(port))
        if username is not None or password is not None:
            transport.username = username
            transport.password = password
        transport.default_from_addr = from_address or 'test@mail.com'
        transport.default_from_name = from_name or 'Test'
        transport.encryption = encryption
        return transport



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt", "r") as fh:
    requirements = fh.readlines()
    requirements = map(lambda x: x.strip(), requirements)
    requirements = filter(lambda x: len(x) > 0, requirements)
    requirements = list(requirements)

print(requirements)

setuptools.setup(name="SNSNotifications",
                 version="0.1.0-1",
                 author="TrinhQuan",
                 author_email="quantv@aimesoft.com",
                 description="This module use for abstract work to send notification a cross sns provider",
                 long_description=long_description,
                 url='https://gitlab.com/aimesoft/libraries/snsnotifications.git',
                 download_url='https://gitlab.com/aimesoft/libraries/snsnotifications/-/archive/master/snsnotifications-master.zip',
                 install_requires=requirements,
                 long_description_content_type="text/plain",
                 packages=setuptools.find_packages(),
                 data_files=['requirements.txt'],
                 classifiers=["Programming Language :: Python :: 3.6", "Programming Language :: Python :: 3.7",
                              "License :: OSI Approved :: MIT License"], )



import smtplib
from email.message import EmailMessage

from ..mailable import Mailable
from ..transport import Transport


class SmtpTransport(Transport):

    def __init__(self, host: str, port: int):
        self.host = host
        self.port = port
        self.username = None
        self.password = None
        self.encryption = False
        self.default_from_addr = None
        self.default_from_name = None

    def send(self, mailable: Mailable):
        if self.encryption:
            smtp = smtplib.SMTP_SSL(self.host, self.port)
        else:
            smtp = smtplib.SMTP(self.host, self.port)
        if self.username:
            smtp.login(self.username, self.password)
        from_addr = mailable.from_addr or self.default_from_addr
        from_name = mailable.from_name or self.default_from_name or from_addr
        to_addrs = mailable.to_addr
        msg = EmailMessage()
        if mailable.reply_to is not None:
            msg.add_header('reply-to', mailable.reply_to)

        msg['subject'] = mailable.subject
        msg['from'] = "{} <{}>".format(from_name, from_addr)
        msg['to'] = mailable.to_addr

        if mailable.cc is not None:
            msg['cc'] = list(mailable.cc)
            to_addrs += list(mailable.cc)
        if mailable.bcc is not None:
            msg['bcc'] = list(mailable.bcc)
            to_addrs += list(mailable.bcc)
        has_attachment = False
        if mailable.attachments is not None:
            for attachment in mailable.attachments:
                has_attachment = True
                content = attachment['content']
                filename = attachment['filename']
                attachment_kwarg = {}
                for (k, v) in attachment.items():
                    if k == 'content' or k == 'filename':
                        continue
                    attachment_kwarg[k] = v
                msg.add_attachment(content, filename=filename, **attachment_kwarg)
        if not has_attachment:
            if mailable.html:
                msg.set_content(mailable.html, subtype='html')
            elif mailable.plain:
                msg.set_content(mailable.plain)
        else:
            if mailable.html:
                msg._add_multipart('mixed', mailable.html, subtype='html')
            elif mailable.plain:
                msg._add_multipart('mixed', mailable.plain, subtype='plain')
        smtp.sendmail(from_addr=from_addr,
                      to_addrs=to_addrs,
                      msg=msg.as_string())



from SNSNotifications.types import NotificationChannel, Notification
from .mailable import Mailable
from .transport_manager import TransportManager

__all__ = ['Mailer', 'Mailable']


class Mailer(NotificationChannel):

    def __init__(self, driver: str, **kwargs):
        self.transport = getattr(TransportManager(), 'create_{}_driver'.format(driver))(**kwargs)

    def send(self, notification: Notification):
        if not self.can_send(notification):
            return
        mail = getattr(notification, "to_{}".format(self.channel_identify()))()
        self.transport.send(mail)

    def channel_identify(self) -> str:
        return "mail"
